@@ -46,7 +46,8 @@ rm(showBlock)
 isInBlock <- function(block, point) {
   length(point) != nrow(block@border) && stop("Point and block dimensions
                                               mismatch")
-  return(all(block@border[,1]<=point & point<=block@border[,2]))
+  res <- all(block@border[,1]-1e-15<=point & point<=block@border[,2]+1e-15)
+  return(res)
 }
 
 getCorners <- function(block) {
@@ -66,8 +67,8 @@ getCorners <- function(block) {
 }
 
 printBorder <- function(block, jaatha) {
-  lower <- .deNormalizeVector(jaatha, block@border[ ,1])
-  upper <- .deNormalizeVector(jaatha, block@border[ ,2])
+  lower <- denormalize(block@border[ ,1], jaatha)
+  upper <- denormalize(block@border[ ,2], jaatha)
   return(paste0(round(lower,3), "-", round(upper,3),
                 collapse=" x "))
 }
