@@ -22,11 +22,13 @@
 #' @param sim The number of simulations that are performed in each step
 #' @param sim.final The number of simulations that are performed after the search to estimate the 
 #'        composite log likelihood. If not specified, the value of \code{sim} will be used
-#' @param epsilon The search stops if the improvement of the score is less than this for 5 times in a row. 
+#' @param epsilon Obsolete. Has no effect anymore and will be remove on next
+#'        major release.
 #' @param half.block.size The size of the new block that is created around a new maximum.
-#' @param weight The weighting factor that will reduce the influence of old block in the estimation procedure
+#' @param weight Obsolete. Has no effect anymore and will be remove on next
+#'        major release.
 #' @param max.steps The search will stop at this number of steps if not stopped
-#' before (see \code{epsilon})
+#'        before (see \code{epsilon}).
 #' @param rerun You can repeat a previously done refined search in Jaatha.
 #'        Do do so, just call the refined search function with the jaatha 
 #'        object result of the first refined search and set rerun to 'TRUE'.
@@ -36,8 +38,11 @@
 #' @export
 Jaatha.refinedSearch <- function(jaatha, best.start.pos=2,
                                  sim=length(getParNames(jaatha))*25,
-                                 sim.final=min(sim, 100), epsilon=2, half.block.size=.025,
-                                 weight=.9, max.steps=200, rerun=FALSE) {
+                                 sim.final=min(sim, 100), epsilon=NULL, half.block.size=.025,
+                                 weight=NULL, max.steps=200, rerun=FALSE) {
+
+  if (!is.null(epsilon)) warning('Parameter "epsilon" is obsolete and will be removed soon.')
+  if (!is.null(weight)) warning('Parameter "weight" is obsolete and will be removed soon.')
 
   if (rerun) {
     if( is.null(jaatha@calls[['refined.search']]) ) 
@@ -46,32 +51,25 @@ Jaatha.refinedSearch <- function(jaatha, best.start.pos=2,
     best.start.pos <- jaatha@calls[['refined.search']]$best.start.pos
     sim <- jaatha@calls[['refined.search']]$sim
     sim.final <- jaatha@calls[['refined.search']]$sim.final
-    epsilon <- jaatha@calls[['refined.search']]$epsilon
     half.block.size <- jaatha@calls[['refined.search']]$half.block.size
-    weight <- jaatha@calls[['refined.search']]$weight
     max.steps <- jaatha@calls[['refined.search']]$max.steps
 
   } else {
     arguments <- list(best.start.pos=best.start.pos,
                       sim=sim,
                       sim.final=sim.final,
-                      epsilon=epsilon,
                       half.block.size=half.block.size,
-                      weight=weight,
                       max.steps=max.steps)
     jaatha@calls[['refined.search']] <- arguments 
   }
 
   # Check parameters
-  .log2("Called function Jaatha.refinedSearch()")
 
   checkType(jaatha, c("jaatha", "single"))
   checkType(best.start.pos, c("num", "single"))
   checkType(sim, c("num", "single"))
   checkType(sim.final, c("num", "single"), F)
-  checkType(epsilon, c("num", "single"))
   checkType(half.block.size, c("num", "single"))
-  checkType(weight, c("num", "single"))
   checkType(max.steps, c("num", "single"))
 
 
@@ -84,7 +82,6 @@ Jaatha.refinedSearch <- function(jaatha, best.start.pos=2,
 
   # Setup environment for the refined search
   set.seed(jaatha@seeds[3])
-  .log2("Seeting seed to", jaatha@seeds[3])
   setParallelization(jaatha@cores)
   tmp.dir <- getTempDir(jaatha@use.shm)
 
@@ -92,8 +89,8 @@ Jaatha.refinedSearch <- function(jaatha, best.start.pos=2,
   for (s in 1:length(startPoints)){
     .print("*** Search with starting Point in Block",s,"of",length(startPoints),"****")
     jaatha <- refinedSearchSingleBlock(jaatha, startPoints[[s]]@MLest, sim=sim,sim.final=sim.final,
-                                        epsilon=epsilon,half.block.size=half.block.size,
-                                        weight=weight,max.steps=max.steps,block.nr=s)  
+                                        half.block.size=half.block.size,
+                                        max.steps=max.steps,block.nr=s)  
   }
 
   .print()
@@ -108,115 +105,87 @@ Jaatha.refinedSearch <- function(jaatha, best.start.pos=2,
 ## This is called from Jaatha.refinedSearch for each block. The actual search is done here.
 ## Parameters are the same as in Jaatha.refinedSearch
 refinedSearchSingleBlock <- function(jaatha, start.point, sim, sim.final,
-                                     epsilon, half.block.size, weight=weight,
+                                     half.block.size, 
                                      max.steps=max.steps, block.nr){
   ## initialize values 
-  .log3("Initializing")
-  nSteps <- 1
-  noLchangeCount <- 0
-  lastNoChange <- -1        # has to be !=0 for the start
-
-  # Track route through parameter space
-  route <- matrix(0, max.steps, getParNumber(jaatha)+1)
-  route[1,  1] <- -1
-  route[1, -1] <- denormalize(start.point, jaatha) 
+  step.current <- 0
 
   ##since the likelihood estimate for the starting point is
   ##only a very rough estimate we don't keep that value 
-  searchBlock <- new("Block", score=-1e11, MLest=start.point)
+  search.block <- new("Block", score=-Inf, MLest=start.point)
   sim.saved <- list()
 
   ## best ten parameters with score are kept for end evaluation
   topTen <- array(0, dim=c(10,(1+getParNumber(jaatha))), 
                   dimnames= list(1:10, c("score", getParNames(jaatha))))     
 
+  optimum.li <- -Inf
+  optimum.step <- 0
+
   repeat{
     .print("-----------------")
-    .print("Step No",nSteps)
+    step.current <- step.current + 1
+    .print("Step No", step.current)
 
-    border <- calcBorders(searchBlock@MLest, radius=half.block.size)
-
-    searchBlock <- new("Block", border=border,
-                       weight=1,
-                       score=searchBlock@score,
-                       MLest=searchBlock@MLest)
+    # Update the Search Blocks Border
+    search.block@border <- calcBorders(search.block@MLest, radius=half.block.size)
 
     # Simulate
-    sim.data <- simulateWithinBlock(sim, searchBlock, jaatha)
-    sim.saved <- getReusableSimulations(searchBlock, jaatha, sim.saved,
-                                        sim.data, nSteps)
+    sim.data <- simulateWithinBlock(sim, search.block, jaatha)
+    sim.saved <- getReusableSimulations(search.block, jaatha, sim.saved,
+                                        sim.data, step.current)
 
-    .print("Using", length(sim.saved), "Simulations")
+    # Fit the GLM
     glm.fitted <- fitGlm(sim.data, jaatha)
 
-    ## likelihood of old newOptimum parameters based on new simulated data              
-    oldParamLikeli <- estimateLogLikelihood(searchBlock@MLest[1:getParNumber(jaatha)], 
-                                            glm.fitted, jaatha@sum.stats)
-
-    optimal <- findBestParInBlock(searchBlock, glm.fitted, jaatha@sum.stats) 
-
-    ## keep the best 10 parameter combinations with their score
-    topTen <- .saveBestTen(currentTopTen=topTen, numSteps=nSteps,
-                           newOptimum=optimal)
-
-    route[nSteps, ] <- c(optimal$score, denormalize(optimal$est, jaatha))
-
-    ## prepare for next round
-    nSteps <- nSteps+1        
-    ## in parNsumstat only the newest simulation results
-    searchBlock@MLest <- optimal$est
-    searchBlock@score <- optimal$score 
-
-    # Output current best position
-    printBestPar(jaatha, searchBlock)
-
-    ## stop criterion 1: likelihood difference less than epsilon 
-    if ( (abs(optimal$score - oldParamLikeli) < epsilon)){
-      #if (lastNoChange==(nSteps-1)){  # if last noChange happend just last step
-      noLchangeCount <- noLchangeCount +1           
-      .print("No sigificant score changes in the last",noLchangeCount,"Step(s)")
-      ## if there has been no change 5 times in a row, stop 
-      if (noLchangeCount>4){
-        .print()
-        .print("*** Finished search ***")
-        .print("Score has not change much in the last 5 steps.")
-        .print("Seems we have converged.")
-        .print()
-        break
-      } else{}
-    } else{ # if first noChange, save old estimates
-      noLchangeCount <- 0
+    # Update likelihood of last steps estimate, based on new simulated data.
+    # Should be a bit more accurate as previous estimate of the likelihood,
+    # as it is in the center of the block now.
+    search.block@score <- estimateLogLikelihood(search.block@MLest, glm.fitted, 
+                                                jaatha@sum.stats)
+    
+    # Keep track of the optimal likelihood, and in which step it has be reached.
+    if (search.block@score > optimum.li) {
+      optimum.li <- search.block@score
+      optimum.step <- step.current - 1
     }
 
-    ## stop criterion 2: more than max.steps search steps
-    if (nSteps>(max.steps-1)) {
+    # Keep the best 10 parameter combinations with their score
+    topTen <- .saveBestTen(topTen, step.current-1, search.block)
+
+    # Output current best position
+    printBestPar(jaatha, search.block)
+
+    # Stop criterion 1: maximal likelihood did not increase in the last 10 steps
+    if ( step.current >= optimum.step + 10 ) {
+      .print()
+      .print("*** Finished search ***")
+      .print("Seems we can not improve the likelihood anymore.")
+      break
+    } 
+
+    # Stop criterion 2: more than max.steps search steps
+    if ( step.current > (max.steps-1) ) {
       .print()
       .print("Maximimum number of search steps",max.steps,"reached.\n")
       .print()
       break
     }
 
+    # Estimate the best parameters in the current block.
+    search.block@MLest <- findBestParInBlock(search.block, glm.fitted, jaatha@sum.stats)$est 
     .print()
   }
 
-  #last best estimates will be taken into topTen
-  nBest <- min(nSteps, nrow(topTen))
-
-  ## inlcude last optimum into top ten
-  topTen[nBest,] <- c(optimal$score, optimal$est)
-
-  route <- route[route[,1] != 0,]
-  jaatha@route[[length(jaatha@route) + 1]] <- route
+  topTen <- topTen[topTen[,1] != 0, ]
 
   likelihoods <- c()
-  .log3("Starting final sim.")
   .print("Calculating log-composite-likelihoods for best estimates:")
-  for (t in 1:nBest){
+  for (t in 1:nrow(topTen)){
     topPar <- topTen[t,-1]
-    .print("* Parameter combination",t,"of",nBest)
+    .print("* Parameter combination",t,"of", nrow(topTen))
     likelihoods[t] <- simLikelihood(jaatha, sim.final, topPar)
   }
-  .log3("Finished final sim.")
 
   likelihood.table <- cbind(log.cl=likelihoods, block=block.nr,topTen[topTen[,1]!=0,-1])
   jaatha@likelihood.table <- rbind(jaatha@likelihood.table,likelihood.table)
@@ -275,7 +244,6 @@ findReusableBlocks <- function(MLpoint, blockList, weighOld, verbose=FALSE){
   if (length(blockList)!=0) {
     for (b in seq(along = blockList)){
       if(isInBlock(blockList[[b]],MLpoint)){
-        blockList[[b]]@weight <- blockList[[b]]@weight*weighOld
         reusableBlocks[[listLen]] <- blockList[[b]]
         if (verbose){ 
           cat("Keeping BLOCK",b," (lower:",round(blockList[[b]]@lowerBound,3),
@@ -305,18 +273,16 @@ findReusableBlocks <- function(MLpoint, blockList, weighOld, verbose=FALSE){
 
 ## Function to save the ten best parameters along the search path with
 ## their likelihoods.
-.saveBestTen <- function (currentTopTen,numSteps,newOptimum){
-  if (numSteps<10){  # the first 9 estimates are kept
-    currentTopTen[numSteps,] <- c(newOptimum$score,newOptimum$est)
-  } else{  # the minimum score in the array is smaller than the new score -> replace
-    minScore <- min(currentTopTen[1:9,1])
-    #print(minScore)
-    if(minScore < newOptimum$score){
-      minIndex <- (1:9) [currentTopTen[,1] == minScore]
-      #cat("minIndex:",minIndex,minScore,"\n")
-      currentTopTen[minIndex,] <- c(newOptimum$score,
-                                    newOptimum$est)
-    }#else{}
+.saveBestTen <- function (currentTopTen, numSteps, search.block){
+  if (numSteps <= 10){  # the first 9 estimates are kept
+    currentTopTen[numSteps, ] <- c(search.block@score, search.block@MLest)
+  } else{
+    minScore <- min(currentTopTen[ ,1])
+    if(minScore < search.block@score){
+      minIndex <- which.min(currentTopTen[ ,1])
+      currentTopTen[minIndex,] <- c(search.block@score,
+                                    search.block@MLest)
+    }
   }
   return(currentTopTen)
 }
